@@ -9,6 +9,7 @@ from box import Box
 
 from . import common
 from . import data
+from .data.validate import must_be_list
 
 def get_config_name(g: dict) -> typing.Optional[str]:
   config_name = g.get('config_name',None)
@@ -25,4 +26,4 @@ def node_config(node: Box, config_name: typing.Optional[str]) -> None:
       node.config =  config + [ config_name ]
 
 def list_attribute(parent: Box, key: str, path: str) -> typing.Optional[list]:
-  return data.must_be_list(parent,key,path)
+  return must_be_list(parent,key,path)
