@@ -13,6 +13,6 @@ class EIGRP(_Module):
       if not _routing.external(intf,'eigrp'):
         _routing.passive(intf,'eigrp')
 
-    _routing.remove_unaddressed_intf(node,'ospf')
-    _routing.remove_vrf_interfaces(node,'isis')
+    _routing.remove_unaddressed_intf(node,'eigrp')
+    _routing.remove_vrf_interfaces(node,'eigrp')
     _routing.routing_af(node,'eigrp')
